import { ChangeDetectorRef, Component, OnInit, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { MatPaginator, PageEvent } from '@angular/material/paginator';
import { MatSort, MatSortModule } from '@angular/material/sort';
import { MatTableDataSource } from '@angular/material/table';
import { AceiteService } from '../../../services/aceites/aceite.service';
import { CoreService } from '../../../core/core.service';
import { VisualizacaoAceiteComponent } from '../visualizacao-aceite/visualizacao-aceite.component';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MatButtonModule } from '@angular/material/button';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatDatepickerModule } from '@angular/material/datepicker';
import { MatSelectModule } from '@angular/material/select';
import { MatRadioModule } from '@angular/material/radio';
import { MatIconModule } from '@angular/material/icon';
import { MatSnackBarModule } from '@angular/material/snack-bar';
import { MatTableModule } from '@angular/material/table';
import { MatToolbarModule } from '@angular/material/toolbar';
import { MatPaginatorModule } from '@angular/material/paginator';
import { AceiteEditComponent } from '../aceite-edit/aceite-edit.component';
import { AceiteAddComponent } from '../aceite-add/aceite-add.component';
import { AddSearchComponent } from '../../../shared/addAndSearch/add-search/add-search.component';
import { AceiteSearchComponent } from '../aceite-search/aceite-search.component';
import { JwtDecoderService } from '../../../jwt-decoder.service';
import { AuthService } from '../../../auth.service';
import { jwtDecode } from 'jwt-decode';
interface AceiteData {
  id: number;
  imo: number;
  codigo: string;
  obs: string;
  status: string;
  bercos: any;
}

@Component({
  selector: 'app-aceites',
  standalone: true,
  imports: [
    MatPaginatorModule,
    MatToolbarModule,
    MatTableModule,
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    MatButtonModule,
    MatInputModule,
    MatFormFieldModule,
    MatDatepickerModule,
    MatSelectModule,
    MatRadioModule,
    MatIconModule,
    MatSnackBarModule,
    MatSortModule,
    AddSearchComponent,
  ],
  templateUrl: './aceites.component.html',
  styleUrls: ['./aceites.component.scss'],
})
export class AceitesComponent implements OnInit {
  displayedColumns = [
    'id',
    'nome',
    'user',
    'imo',
    'data_create',
    'data_update',
    'dataAccept',
    'status',
    'action',
  ];

  dataSource = new MatTableDataSource<AceiteData>();
  totalItems = 100;
  pageSize = 10;
  currentPage = 1;
  user: any;
  authTokenPresent = false;

  @ViewChild(MatPaginator) paginator!: MatPaginator;
  @ViewChild(MatSort) sort!: MatSort;

  constructor(
    private _dialog: MatDialog,
    private _aceiteService: AceiteService,
    private _coreService: CoreService,
    private jwtDecoderService: JwtDecoderService,
    private authService: AuthService,
    private cdr: ChangeDetectorRef

  ) {}

  ngOnInit(): void {    this.validationToken();
  }

  ngAfterViewInit(): void {
    this.loadData();
  }
  parseUserString(userString: string) {
    const userRegex = /User\(id=(\d+), email=([^)]+)\)/;
    const match = userRegex.exec(userString);
    if (match) {
      return {
        id: parseInt(match[1], 10),
        email: match[2],
      };
    }
    return null;
  }
  loadData(event?: PageEvent): void {
    if (typeof sessionStorage !== 'undefined') {
      const storedData = sessionStorage.getItem('aceitesSearch');
      if (storedData) {
        const aceites = JSON.parse(storedData); // Converte o JSON em um objeto
        console.log('Dados recuperados do sessionStorage:', aceites);
  
        // Atualiza os dados na tabela
        this.dataSource.data = aceites;
  
        // Configurações do paginator e sort
        if (this.paginator && this.sort) {
          this.dataSource.paginator = this.paginator;
          this.dataSource.sort = this.sort;
        }
  
        // Atualiza a visualização
        this.cdr.detectChanges();
  
        // Remove dados do sessionStorage após o uso
        sessionStorage.removeItem('aceitesSearch');
        console.log('aceitesSearch removido do sessionStorage.');
  
        return; // Retorna para evitar chamada à API
      } else {
        console.warn('Nenhum dado encontrado no sessionStorage.');
      }
    }
  
    // Caso os dados não estejam no sessionStorage, chama a API
    const pageIndex = event ? event.pageIndex + 1 : this.currentPage - 1;
  
    this._aceiteService.getAceiteList(pageIndex, this.pageSize).subscribe({
      next: (res: any) => {
        const aceites = res._embedded.acceptResponseList;
        console.log(aceites);
        this.dataSource.data = aceites;
        this.totalItems = res.page.totalElements;
        console.log(res.page.totalElements);
      },
      error: (err: any) => {
        console.error(err);
        if (err.error.title) {
          this._coreService.openSnackBar(err.error.title);
        } else {
          this._coreService.openSnackBar(err.error.message);
        }
      },
    });
  }

  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();

    if (this.dataSource.paginator) {
      this.dataSource.paginator.firstPage();
    }
  }

  deleteBerco(id: number) {
    this._aceiteService.deleteEmployee(id).subscribe({
      next: () => {
        this._coreService.openSnackBar('Aceite deleted!', 'done');
        this.loadData();
      },
      error: (err: any) => {
        this._coreService.openSnackBar(err.error.title);
      },
    });
    this.cdr.detectChanges();

  }

  openEditForm(data: any) {
    this._dialog.open(AceiteEditComponent, {
      data,
      autoFocus: true, // Foca no primeiro campo interativo do formulário
      disableClose: true, // Impede que o diálogo feche ao clicar fora
      width: '600px', // Define a largura do diálogo
    });
    this.cdr.detectChanges();
  }

  openSearchBercoForm() {
    this._dialog.open(AceiteSearchComponent, {
      autoFocus: true, // Foca no primeiro campo interativo do formulário
      disableClose: true, // Impede que o diálogo feche ao clicar fora
      width: '600px' // Define a largura do diálogo
    });
  
    // Verifica se o sessionStorage está disponível
    if (typeof sessionStorage !== 'undefined') {
      const storedData = sessionStorage.getItem('aceitesSearch');
      if (storedData) {
        const dataSource = JSON.parse(storedData); // Converte o JSON em um objeto
        console.log('Dados recuperados do sessionStorage:', dataSource);
        this.dataSource.data = dataSource;
        this.cdr.detectChanges();
        // Atualiza a tabela com os dados
      } else {
        console.warn('Nenhum dado encontrado no sessionStorage.');
      }
    } else {
      console.error('sessionStorage não está disponível.');
    }
  }
  
  //
  openAddBercoForm() {
    this._dialog.open(AceiteAddComponent,{autoFocus: true, // Foca no primeiro campo interativo do formulário
      disableClose: true, // Impede que o diálogo feche ao clicar fora
      width: '600px', // Define a largura do diálogo});
  });}

  getBercoById(data: any) {
    this._dialog.open(VisualizacaoAceiteComponent, {
      data,
      autoFocus: true, // Foca no primeiro campo interativo do formulário
      width: '600px', // Define a largura do diálogo
    });
  }
  validationToken() {
    this.authService.authToken$.subscribe((isLoggedIn) => {
      this.authTokenPresent = isLoggedIn;
      if (this.authTokenPresent) {
        const token = sessionStorage.getItem('accessToken')!;
        this.user = jwtDecode(token);
        const isExpired =
          this.user && this.user.exp
            ? this.user.exp < Date.now() / 1000
            : false;
        if (isExpired) {
          sessionStorage.removeItem('accessToken');
        }
        this.user = this.jwtDecoderService.decodeToken(token);
        console.log(this.user);
      }
      this.cdr.detectChanges();
    });
  }
}
